{% extends "admin/change_list.html" %}
{% load admin_static i18n admin_modify %}

{% block object-tools %}
    {% if has_add_permission %}
        <ul class="grp-object-tools">
            {% block object-tools-items %}
                {{ block.super }}
            {% endblock %}
            <li><a href="{{ unisender_site }}" class="grp-state-focus">Просмотр сообщений на сайте unisender</a></li>
        </ul>
    {% endif %}
{% endblock %}
