{% extends "admin/change_form.html" %}

<!-- LOADING -->
{% load i18n %}


<!-- OBJECT TOOLS -->
{% block object-tools-items %}
    {{ block.super }}
    {% if unisender_site %}
        <li><a href="{{ unisender_site }}" class="grp-state-focus">Просмотр сообщения на сайте unisender</a></li>
    {% endif %}
{% endblock %}
