# -*- coding: utf-8 -*-

UNISENDER_COMMON_ERRORS = {
    'unspecified': u'Тип ошибки не указан. Подробности смотрите в сообщении.',
    'invalid_api_key': u'Указан неправильный ключ доступа к API. Проверьте, совпадает ли значение api_key со значением, указанным в личном кабинете.',
    'access_denied': u'Доступ запрещён. Проверьте, включён ли доступ к API в личном кабинете и не обращаетесь ли вы к методу, прав доступа к которому у вас нет.',
    'unknown_method': u'Указано неправильное имя метода',
    'invalid_arg': u'Указано неправильное значение одного из аргументов метода',
    'not_enough_money': u'Не хватает денег на счету для выполнения метода',
    'api_call_limit_exceeded_for_api_key': u'Сработало ограничение по вызову методов API в единицу времени. На данный момент это 300 вызовов в минуту.',
    'api_call_limit_exceeded_for_ip': u'Сработало ограничение по вызову методов API в единицу времени. На данный момент это 300 вызовов в минуту.',
}
