# -*- coding: utf-8 -*-

EMAIL_MESSAGES_LIST = 'http://cp.unisender.com/ru/v5/letter/list'
EMAIL_MESSAGES_DETAIL = 'http://cp.unisender.com/ru/v5/letter/send/'

TAG_LIST = 'http://cp.unisender.com/ru/v5/tags'

FIELD_LIST = 'http://cp.unisender.com/ru/v5/contact/field/list'

CAMPAIGN_LIST = 'http://cp.unisender.com/ru/v5/bursts/reports-list'
CAMPAIGN_DETAIL = 'http://cp.unisender.com/ru/v5/bursts/view/'

SUBSCRIBELIST_LIST = 'http://cp.unisender.com/ru/v5/lists'
SUBSCRIBELIST_DETAIL = 'http://cp.unisender.com/ru/v5/contact/subscriber/list/'
